import { BadRequestException, Injectable } from "@nestjs/common";
import * as argon from "argon2";
import * as jose from "jose"
// biome-ignore lint/style/useImportType: used by Next.js
import { ConfigService } from "@nestjs/config";

@Injectable()
export class CredentialsService {

	constructor(private configService: ConfigService) {
	}

	async hash(plaintextPassword: string) {
		if (plaintextPassword.length < 6) throw new BadRequestException("Password is not strong enough !")
		return argon.hash(Buffer.from(plaintextPassword), {
			secret: this.configService.get("APP_HASH_SECRET"),
		})
	}

	async check(plaintextPassword: string, hashedPassword: string) {
		return argon.verify(hashedPassword, Buffer.from(plaintextPassword), {
			secret: this.configService.get("APP_HASH_SECRET"),
		})
	}

	async verifyAuthToken(token: string) {
		const verifyRes = await jose.jwtVerify(token, Uint8Array.from(this.configService.get("APP_TOKEN_SECRET")), {
			subject: "auth",
			audience: "user",
			issuer: "ShouldStick"
		})
	}

	async signAuthToken() {
		return new jose.SignJWT({})
			.setProtectedHeader({ alg: 'dir', enc: 'A128CBC-HS256' })
			.setIssuedAt()
			.setExpirationTime('3 day')
			.setIssuer("ShouldStick")
			.setAudience("user")
			.setSubject("auth")
			.sign(Uint8Array.from(this.configService.get("APP_TOKEN_SECRET")))
	}
}
